function [localisationIdx, assignedClass, imgDirList, Nreps, doTraining] = label_samples(rcfd, imageFileExtension, folderList, goodClass, badClass, unknownClass, Nreps, maxBright, minBright, localisationIdxIn, samplingWeight)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Called by run_training.m
%    GUI interface so the user can label a bunch of localisations. These are then used to train the classifier
%    Inputs:
%        goodClass - int, value indicating a good localisations
%        badClass - int, value indicating a too dense localisation
%        unknownClass - int, value indicating a background localisation
%        folderList - struct, list of folders in which the raw images are stored
%	 imageFileExtension - string, extension of image files, e.g. '.tiff'
%        rcfd - NX4 matrix, contains the row co-ord, column co-ord, frame number and directory number of each localisations
%        maxBright - number, brightest pixel encountered
%        minBright - number, dimmest pixel encountered
%        Nreps - int, number of samples to be labelled
%        localisationIdxIn - vector, index of samples to be labelled. If not exist, these are drawn using stratified sampling
%        samplingWeight - NX1 vector, probability of each sample being labelled
%    Outputs:
%        localisationIdx - NrepsX1 vector, indicies of labelled samples
%        assignedClass - NrepsX1 vector, labels assigned to each localisation indicated in localisationIdx
%        imgDirList - struct, list of image locations
%        Nreps - int, numebr of labelled samples
%        doTraining - bool, whether the user hit quit or not
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



	%figCurrent = figure();
	Nreps = min(Nreps, size(rcfd,1));
	localisationIdx = zeros(Nreps,1);
	assignedClass = localisationIdx;
	doTraining = true;
	currentFolder = pwd;
	colMaps = {parula, jet, hsv, hot, cool, spring, summer, autumn, winter, gray, bone, copper, pink, lines, colorcube, prism, flag, white};
	colmapIndex = 4;

	imgDirList = [];
	for d=1:length(folderList),
		cd(folderList(d).name);
		imgDirList(d).imgDir = dir(imageFileExtension);
		cd(currentFolder);
	end
	


	if not(exist('samplingWeight')),
		samplingWeight = ones(size(rcfd,1),1);
	end




	

	if not(exist('localisationIdxIn')),
		localisationIdxIn = stratisample(rcfd(:,3:4), Nreps, samplingWeight);
		%localisationIdxIn = randsample(size(rcf,1),Nreps,false);
	end


	uniqueD = unique(rcfd(:,4));
	for i =1:length(uniqueD),

		reconimage(i).img = accumarray(ceil(rcfd(rcfd(:,4)==uniqueD(i),1:2)), 1, [ceil(max(rcfd(rcfd(:,4)==uniqueD(i),1))) , ceil(max(rcfd(rcfd(:,4)==uniqueD(i),2)))]     );

	end

	

	margin = 10;

	patchDisplaySize = 63;
	patchDisplayH = 50;
	patchDisplayV = 50;

	imgDisplaySize = 192;
	imgDisplayH = patchDisplayH;
	imgDisplayV = patchDisplayV + patchDisplaySize + imgDisplaySize/2;


	buttonH = patchDisplayH + 3*patchDisplaySize+40;
	buttonV = patchDisplayV;
	buttonWidth = 100;
	buttonHeight = 25;

	textWidth = 100;
	textWidthBig = 150;
	textHeight = 25;


	patchesPanelSize = [patchDisplayH, patchDisplayV, margin+patchDisplaySize+margin+textWidth+margin, margin + 3*(patchDisplaySize+margin) + 20];
	imgPanelSize = [patchesPanelSize(1), patchesPanelSize(2)+patchesPanelSize(4)+margin, margin + 2*(imgDisplaySize +margin)+20, margin+imgDisplaySize+margin+textHeight+margin+20];

	ButtonsPatchSize = [patchDisplayH +  margin+patchDisplaySize+margin+textWidth+margin + margin, patchDisplayV + margin + textHeight + margin + 20, margin+buttonWidth+margin, margin + 3*(buttonHeight+margin) + 20];
	progressPatchSize = [patchDisplayH +  margin+patchDisplaySize+margin+textWidth+margin + margin, patchDisplayV , margin+textWidth+margin, margin + textHeight + margin + 20];

	quitPatchSize = [progressPatchSize(1)+progressPatchSize(3)+margin, progressPatchSize(2), margin+buttonWidth+margin, patchesPanelSize(4)];


	figWidth = max([(patchesPanelSize(1)+patchesPanelSize(3));(imgPanelSize(1)+imgPanelSize(3));(ButtonsPatchSize(1)+ ButtonsPatchSize(3)); (progressPatchSize(1) + progressPatchSize(3))])+50;
	figHeight = max([(patchesPanelSize(2)+patchesPanelSize(4));(imgPanelSize(2)+imgPanelSize(4));(ButtonsPatchSize(2)+ ButtonsPatchSize(4)); (progressPatchSize(2) + progressPatchSize(4))])+50;
	scsz = get(0,'ScreenSize');
	fig = figure('Visible','on','Position',[scsz(1)+10,scsz(4)-figHeight-10,figWidth, figHeight]);

	hpatchesPanel = uipanel('Title','Patches','FontSize',12,'Units','pixels', 'BackgroundColor','white', 'Position',patchesPanelSize);

	PriorPatchH = margin;
	PriorPatchV = margin;
	hPriorPatch = axes('Parent', hpatchesPanel, 'Units','pixels','Position',[PriorPatchH,PriorPatchV,patchDisplaySize,patchDisplaySize]);
	PriorPatchTextH = PriorPatchH+margin+patchDisplaySize;
	PriorPatchTextV = PriorPatchV+(patchDisplaySize-textHeight)/2;
	hPriorPatchText = uicontrol('Parent', hpatchesPanel, 'Style','text','String','t-1','Position',[PriorPatchTextH,PriorPatchTextV,textWidth,textHeight]);

	patchH = PriorPatchH;
	patchV = PriorPatchV+margin+patchDisplaySize
	hPatch = axes('Parent', hpatchesPanel, 'Units','pixels','Position',[patchH,patchV,patchDisplaySize,patchDisplaySize]);
	patchTextH = patchH+margin+patchDisplaySize;
	patchTextV = patchV+(patchDisplaySize-textHeight)/2;
	hPatchText = uicontrol('Parent', hpatchesPanel, 'Style','text','String','Patch, time = t','Position',[patchTextH,patchTextV,textWidth,textHeight]);

	PostPatchH = patchH;
	PostPatchV = patchV+margin+patchDisplaySize;
	hPostPatch = axes('Parent', hpatchesPanel, 'Units','pixels','Position',[PostPatchH,PostPatchV,patchDisplaySize,patchDisplaySize]);
	PostPatchTextH = PostPatchH+margin+patchDisplaySize;
	PostPatchTextV = PostPatchV+(patchDisplaySize-textHeight)/2;
	hPostPatchText = uicontrol('Parent', hpatchesPanel, 'Style','text','String','t+1','Position',[PostPatchTextH,PostPatchTextV,textWidth,textHeight]);




	hImgsPanel = uipanel('Title','Full Images','FontSize',12,'Units','pixels', 'BackgroundColor','white', 'Position',imgPanelSize);
	



	imgH = margin;
	imgV = margin +textHeight+margin;
	hImg = axes('Parent',hImgsPanel,'Units','pixels','Position',[imgH,imgV,imgDisplaySize,imgDisplaySize]);
	hImgTextH = imgH 
	hImgTextV = margin
	hImgText = uicontrol('Parent',hImgsPanel,'Style','text','String','Log image brightness','Position',[hImgTextH,hImgTextV,textWidthBig,textHeight]);

	surfH = imgH + imgDisplaySize + margin + 20;
	surfV = imgV;
	hsurf = axes('Parent',hImgsPanel,'Units','pixels','Position',[surfH,surfV,imgDisplaySize,imgDisplaySize]);
	hsurfTextH = surfH;
	hsurfTextV = hImgTextV;
	hsurfText = uicontrol('Parent',hImgsPanel,'Style','text','String','image patch surface','Position',[hsurfTextH,hsurfTextV,textWidthBig,textHeight]);



	hButtonsPatch = uipanel('Title','Classify patch:','FontSize',12,'Units','pixels', 'BackgroundColor','white', 'Position',ButtonsPatchSize);


	hgoodH = margin;
	hgoodV = margin;
	hgood = uicontrol('Parent', hButtonsPatch, 'Style','pushbutton','String','accurate','Position', [hgoodH,hgoodV,buttonWidth,buttonHeight], 'Callback',@goodButton_Callback);

	hbackgroundH = hgoodH;
	hbackgroundV = hgoodV + buttonHeight + margin;
	hbackground = uicontrol('Parent', hButtonsPatch, 'Style','pushbutton','String','background','Position', [hbackgroundH,hbackgroundV,buttonWidth,buttonHeight], 'Callback',@backgroundButton_Callback);

	hdenseH = hbackgroundH;
	hdenseV = hbackgroundV + buttonHeight + margin;
	hdense = uicontrol('Parent', hButtonsPatch, 'Style','pushbutton','String','dense','Position', [hdenseH,hdenseV,buttonWidth,buttonHeight], 'Callback',@denseButton_Callback);

	hprogressPatch = uipanel('Title','progress','FontSize',12,'Units','pixels', 'BackgroundColor','white', 'Position',progressPatchSize);
	hprogressH = margin;
	hprogressV = margin;
	hprogress = uicontrol('Parent',hprogressPatch,'Style','text','String',[num2str(0), ' / ', num2str(Nreps)],'Position',[hprogressH,hprogressV,textWidth,textHeight]);



	hquitPatch = uipanel('Title','Other controls','FontSize',12,'Units','pixels', 'BackgroundColor','white', 'Position',quitPatchSize);
	hquitButtonH = margin;
	hquitButtonV = margin;
	hquitButton = uicontrol('Parent', hquitPatch, 'Style','pushbutton','String','quit','Position', [hquitButtonH,hquitButtonV,buttonWidth,buttonHeight], 'Callback',@quitButton_Callback);
	hquitAndClassifyH = hquitButtonH;
	hquitAndClassifyV = hquitButtonV + buttonHeight + margin;
	hquitAndClassifyButton = uicontrol('Parent', hquitPatch, 'Style','pushbutton','String','quit and classify','Position', [hquitAndClassifyH,hquitAndClassifyV,buttonWidth,buttonHeight], 'Callback',@quitAndClassifyButton_Callback);
	hcolMapH = hquitAndClassifyH;
	hcolMapV = hquitAndClassifyV + buttonHeight + margin + 50;
	hcolMapButton = uicontrol('Parent', hquitPatch, 'Style','pushbutton','String','set color map','Position', [hcolMapH,hcolMapV,buttonWidth,buttonHeight], 'Callback',@colMapButton_Callback);
	hbackButtonH = hcolMapH;
	hbackButtonV = hcolMapV + buttonHeight + margin;
	hbackButton = uicontrol('Parent', hquitPatch, 'Style','pushbutton','String','go back a patch','Position', [hbackButtonH,hbackButtonV,buttonWidth,buttonHeight], 'Callback',@backButton_Callback);
	hskipButtonH = hbackButtonH;
	hskipButtonV = hbackButtonV + buttonHeight + margin;
	hskipButton = uicontrol('Parent', hquitPatch, 'Style','pushbutton','String','skip patch','Position', [hskipButtonH,hskipButtonV,buttonWidth,buttonHeight], 'Callback',@skipButton_Callback);






	


	set(fig,'Units', 'normalized');

	set(hpatchesPanel,'Units', 'normalized');
	set(hPatch,'Units', 'normalized');
	set(hPatchText,'Units', 'normalized');
	set(hPriorPatch,'Units', 'normalized');
	set(hPriorPatchText,'Units', 'normalized');
	set(hPostPatch,'Units', 'normalized');
	set(hPostPatchText,'Units', 'normalized');

	set(hImgsPanel,'Units', 'normalized');
	set(hImg,'Units', 'normalized');
	set(hImgText,'Units', 'normalized');
	set(hsurf,'Units', 'normalized');
	set(hsurfText,'Units', 'normalized');

	set(hButtonsPatch,'Units', 'normalized');
	set(hgood,'Units', 'normalized');
	set(hbackground,'Units', 'normalized');
	set(hdense,'Units', 'normalized');

	set(hprogress,'Units', 'normalized');
	set(hprogressPatch,'Units', 'normalized');

	set(hquitPatch,'Units', 'normalized');
	set(hquitButton,'Units', 'normalized');
	set(hquitAndClassifyButton,'Units', 'normalized');
	set(hcolMapButton,'Units', 'normalized');
	set(hbackButton,'Units', 'normalized');
	set(hskipButton,'Units', 'normalized');

	earlyQuit = false;
	quitAt = -1;

	n = 1;
	skipArray = [];
	while n<=Nreps,


		%pick a random activation from this
	
		selectedLocalisation = localisationIdxIn(n);
	
		r = rcfd(selectedLocalisation,1);
		c = rcfd(selectedLocalisation,2);
		f = rcfd(selectedLocalisation,3);
		d = rcfd(selectedLocalisation,4);	
	
	
	
		%get image to look at
		cd(folderList(d).name);
		img = double(imread(imgDirList(d).imgDir(f).name));
	
		maximg = max(img(:));
		minimg = min(img(:));

		%only going to show a patch
		patchsize = 21;
		rmin = max(r-(patchsize-1)/2,1);
		rmax = min(r+(patchsize-1)/2,size(img,1));
		cmin = max(c-(patchsize-1)/2,1);
		cmax = min(c+(patchsize-1)/2,size(img,2));

		imgpatch = img(rmin:rmax, cmin:cmax);
		
		img(rmin:rmax,cmin:cmax) = img(rmin:rmax,cmin:cmax)*1.2;

		%get prior and posterio images if possible
		
		if f>1,
			prior = double(imread(imgDirList(d).imgDir(f-1).name));
			prior = prior(rmin:rmax, cmin:cmax);
		else
			prior = ones(1+rmax-rmin, 1+cmax-cmin)*minimg;
		end
	
		if f<length(imgDirList(d).imgDir)
			posterior = double(imread(imgDirList(d).imgDir(f+1).name));
			posterior = posterior(rmin:rmax, cmin:cmax);
		else
			posterior = ones(1+rmax-rmin, 1+cmax-cmin)*minimg;
		end
	


		cd(currentFolder);
	


		hold(hPriorPatch, 'off'); 	imshow((prior-minimg)/(maximg-minimg),     'Parent', hPriorPatch);
		hold(hPatch, 'off'); 		imshow((imgpatch-minimg)/(maximg-minimg),  'Parent', hPatch);
		hold(hPostPatch, 'off'); 	imshow((posterior-minimg)/(maximg-minimg), 'Parent', hPostPatch);

		hold(hImg,'off'); 		imshow(log(1+100*(img-minBright)/(maxBright-minBright))/log(101), 'Parent', hImg);
		hold(hImg,'on'); 		plot(c,r,'.', 'Parent', hImg);
						plot([cmin;cmin], [rmin;rmax], 'k', 'Parent', hImg);
						plot([cmin;cmax], [rmax;rmax], 'k', 'Parent', hImg);
						plot([cmax;cmax], [rmax;rmin], 'k', 'Parent', hImg);
						plot([cmax;cmin], [rmin;rmin], 'k', 'Parent', hImg);

		%hold(hRecon,'off'); 		imshow(reconimage(find(uniqueD==d)).img,[],'Parent', hRecon);
		%hold(hRecon,'on'); 		plot(c,r,'.','Parent', hRecon); 		

		hold(hsurf,'off'); surf(imgpatch, 'Parent', hsurf);

		set(hprogress, 'String', [num2str(n), ' / ', num2str(Nreps)])

		colormap(colMaps{colmapIndex});

		uiwait;


		if earlyQuit,
			break;
		end

		localisationIdx(n) = selectedLocalisation;
		assignedClass(n) = inputClass;


		n = n+1;
	end



	if earlyQuit,
		localisationIdx = localisationIdx(1:quitAt);
		assignedClass = assignedClass(1:quitAt);
		Nreps = quitAt;
	end

	if length(skipArray)>0,
		localisationIdx(skipArray) = [];
		assignedClass(skipArray) = [];
		Nreps = length(assignedClass);
	end

	close(fig);


	%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	function backButton_Callback(source,eventdata)
		if n>1,
			n = n-2;
			uiresume;
		end

	end

	function skipButton_Callback(source,eventdata)
		skipArray = [skipArray;n];
		uiresume;
	end

	function quitButton_Callback(source,eventdata)
		doTraining = false;
		earlyQuit = true;
		quitAt = n-1;
		uiresume;
	end


	function quitAndClassifyButton_Callback(source,eventdata)

		earlyQuit = true;
		quitAt = n-1;
		uiresume;

	end

	function goodButton_Callback(source,eventdata)
		inputClass = goodClass;
		uiresume;
	end

	function backgroundButton_Callback(source,eventdata)
		inputClass = badClass;
		uiresume;
	end

	function denseButton_Callback(source,eventdata)
		inputClass = unknownClass;
		uiresume;
	end

	function colMapButton_Callback(source,eventdata)
		mapsS = {'parula', 'jet', 'hsv', 'hot', 'cool', 'spring', 'summer', 'autumn', 'winter', 'gray', 'bone', 'copper', 'pink', 'lines', 'colorcube', 'prism', 'flag', 'white'};
		[s,v] = listdlg('PromptString','Select color map to display:', 'SelectionMode','single', 'ListString',mapsS);
		if v==0,
		

		else
			colmapIndex = s;
			colormap(colMaps{colmapIndex});
		end;
	end

end







